@@ -1,8 +1,12 @@
+"""
+CalculatorData
+---------
+A data model for creating subclasses to process timeline followback and visit data in the abstinence calculation
+"""
+
 from enum import Enum
-from pathlib import Path
 import pandas as pd
-from io import BytesIO
-from abstcal.calculator_error import InputArgumentError, FileFormatError, FileExtensionError, _show_warning
+from abstcal.calculator_error import InputArgumentError, FileFormatError
 
 
 class DataImputationCode(Enum):
@@ -17,6 +21,11 @@ class CalculatorData:
     subject_ids: set
     _index_keys: list
     _value_key: str
+    use_raw_date: bool
+
+    @property
+    def col_names(self):
+        return [*self._index_keys, self._value_key]
 
     def check_duplicates(self, duplicate_kept="mean"):
         """
@@ -66,65 +75,14 @@ class CalculatorData:
             self.data.reset_index(drop=True)
         return missing_records
 
-    @staticmethod
-    def read_data_from_path(filepath):
-        """
-        Read data from the specified path
-
-        :param filepath: Union[str, Path, BytesIO], the path to the file to be read
-            Supported file types: comma-separated, tab-separated, and Excel spreadsheet
-
-        :return: a DataFrame
-        """
-        path = Path(filepath)
-        file_extension = path.suffix.lower()
-        if file_extension == ".csv":
-            df = pd.read_csv(filepath, infer_datetime_format=True)
-        elif file_extension in (".xls", ".xlsx", ".xlsm", ".xlsb"):
-            df = pd.read_excel(filepath, infer_datetime_format=True)
-        elif file_extension == ".txt":
-            df = pd.read_csv(filepath, sep='\t', infer_datetime_format=True)
-        else:
-            raise FileExtensionError(filepath)
-        return df
-
-    @staticmethod
-    def write_data_to_path(df, filepath, index=False):
-        """
-        Write data to the specified path
-
-        :param df: DataFrame
-        :param filepath: Union[str, Path, None], the path to the file to be read
-            Supported file types: comma-separated, tab-separated, and Excel spreadsheet, if None, no writing will be
-            performed
-
-        :param index: bool, whether the index of the DataFrame will be written to the output file
-
-        :return: a DataFrame
-        """
-        if filepath is None:
-            return
-
-        path = Path(filepath)
-        file_extension = path.suffix.lower()
-        if file_extension == ".csv":
-            pd.DataFrame.to_csv(df, path, index=index)
-        elif file_extension in (".xls", ".xlsx", ".xlsm", ".xlsb"):
-            pd.DataFrame.to_excel(df, path, index=index)
-        elif file_extension == ".txt":
-            pd.DataFrame.to_csv(df, path, sep='\t', index=index)
-        else:
-            raise FileExtensionError(filepath)
-
-    @staticmethod
-    def _validate_columns(df, needed_cols_ordered, data_name, col_names):
-        needed_cols_unordered = set(needed_cols_ordered)
+    def validate_data(self, df):
+        needed_cols_unordered = set(self.col_names)
         current_cols_unordered = set(df.columns)
-        if len(current_cols_unordered) != len(needed_cols_unordered):
-            raise FileFormatError(f'The {data_name} data should have only {col_names} columns.')
-        if needed_cols_unordered != current_cols_unordered:
-            warning_message = f"The {data_name} data does not appear to have the needed columns: {col_names}. " \
-                              f"It has been renamed in such order for calculation purposes. If your columns are not " \
-                              f"in this order, and you'll encounter errors in later calculation steps."
-            _show_warning(warning_message)
-            df.columns = needed_cols_ordered
+        if needed_cols_unordered.issubset(current_cols_unordered):
+            if self.use_raw_date:
+                df['date'] = pd.to_datetime(df['date'], infer_datetime_format=True)
+            if self._value_key == 'amount':
+                df['amount'] = df['amount'].astype(float)
+            return df.loc[:, self.col_names]
+        else:
+            raise FileFormatError(f'Required Columns of {self.__class__.__name__}: {self.col_names}')
